--[[
LuCI - Lua Configuration Interface

Copyright 2008 Steven Barth <steven@midlink.org>
Copyright 2008 Jo-Philipp Wich <xm@leipzig.freifunk.net>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

$Id$
]]--

module("luci.controller.vpn", package.seeall)
require("uci")
function index()
	local fs = require "nixio.fs"
	page = entry({"admin", "services", "VPN", "reconnect"}, call("reconnect"), nil)
	page.leaf = true
	page = entry({"admin", "services", "VPN", "shutdown"}, call("shutdown"), nil)
	page.leaf = true
	page = entry({"admin", "services", "VPN", "iface_status"}, call("iface_status"), nil)
	page.leaf = true
	page = entry({"admin", "services", "VPN", "iface_delete"}, call("iface_delete"), nil)
	page.leaf = true	
	page = entry({"admin", "services", "VPN"},cbi("vpn/vpn"), _("VPN"), 10)
        --entry({"admin", "services", "VPN", "IPSEC_SETTING"}, cbi("vpn/ipsec", {hideapplybtn=true,hidesavebtn=true,hideresetbtn=true}), nil)
        entry({"admin", "services", "VPN", "IPSEC_SETTING_g_to_g"}, cbi("vpn/ipsec_g_to_g", {hideapplybtn=true,hidesavebtn=true,hideresetbtn=true}), nil).leaf = true
        entry({"admin", "services", "VPN", "IPSEC_SETTING_c_to_g"}, cbi("vpn/ipsec_c_to_g", {hideapplybtn=true,hidesavebtn=true,hideresetbtn=true}), nil).leaf = true
        entry({"admin", "services", "VPN", "IPSEC"},template("vpn/ipsec_ovrw"),"IPSEC",2)
        entry({"admin", "services", "VPN", "IPSEC_CHOOSE"},template("vpn/ipsec_choose"),nil)
        entry({"admin", "services", "VPN", "PPTP"},template("vpn/pptp_overview"),"PPTP",1)
        entry({"admin", "services", "VPN", "PPTP_SETTING"}, cbi("vpn/proto_pptp",{hideapplybtn=false,hidesavebtn=false,hideresetbtn=false} ),nil)
        if fs.access("/usr/lib/lua/luci/model/cbi/vpn/l2tp.lua") then
        	entry({"admin", "services", "VPN", "L2TP"},template("vpn/l2tp_overview"),"L2TP",4)
        	entry({"admin", "services", "VPN", "L2TP_SETTING"}, cbi("vpn/l2tp",{hideapplybtn=true,hidesavebtn=true,hideresetbtn=true} ),nil)
        end   
        if fs.access("/usr/lib/lua/luci/model/cbi/vpn/gre.lua") then
        	entry({"admin", "services", "VPN", "GRE"},template("vpn/gre_overview"),"GRE",5)
        	entry({"admin", "services", "VPN", "GRE_SETTING"}, cbi("vpn/gre",{hideapplybtn=true,hidesavebtn=true,hideresetbtn=true} ),nil)
        end   
        entry({"admin", "services", "VPN", "ipsec_status"}, call("action_clock_status")) 
        entry({"admin", "services", "VPN", "ipsecovrw_status"}, call("action_status")) 
        entry({"admin", "services", "VPN","ipsec_param"},
        	call("ipsec_update"), nil).leaf=true
    end

function ipsec_update(cmd,connname,configname)
    local rv = { } 
	luci.sys.exec("echo start %q >> /tmp/viktst" %cmd)
    if cmd == "StartSrv" or cmd == "Connect" then
        luci.sys.exec("uci set ipsec.%q.enable=1 && uci commit ipsec" %configname)
    end
    if cmd == "StopSrv" or cmd == "Disconnect" then
        luci.sys.exec("uci set ipsec.%q.enable=0 && uci commit ipsec" %configname)
    end
	comd="/sbin/ipsec1 "..cmd.." "..connname.." "..configname
	luci.sys.exec(comd)  
    luci.http.prepare_content("application/json")                                                                
    luci.http.write_json(rv)    
end

function action_status()
    	local i=1                                                                                       
    	local rv = { }  
    	x = uci.cursor()
	x:foreach("ipsec", "ipsec", function(s)
       		for key, value in pairs(s) do
		        if key==".name" then
		                local data = {                                                                                                                                       
                			fqdn = luci.sys.exec("cat /tmp/ipsec_%q/status" % value),                                                                                             
			                abc=luci.sys.exec("cat /tmp/ipsec_%q/params" % value),
				}
			rv[i] = data
			i = i + 1
			end
        	end
	 end
	)
    luci.http.prepare_content("application/json")                                                                
    luci.http.write_json(rv)  
end

function action_clock_status()
	luci.http.prepare_content("application/json")
	luci.http.write_json({ dot = luci.sys.exec("cat /tmp/ipsectest") })
end

function reconnect(iface)
	local netmd = require "luci.model.network".init()
	local net = netmd:get_network(iface)
	luci.sys.call("echo hello > /tmp/hello")
	local cmp=iface:match("openvpn")
	if net then
		if cmp == "openvpn" then
		luci.sys.exec("val=`uci show openvpn | grep %q | grep val | cut -d . -f2` ; /etc/init.d/openvpn start $val" % iface)	
		end
		luci.sys.exec("uci set network.%q.stop=no && uci commit network" % iface)
		luci.sys.call("env -i /sbin/ifup %q >/dev/null 2>/dev/null" % iface)
		luci.http.status(200, "Reconnected")
		return
	end
	luci.http.status(404, "No such interface")
end

function shutdown(iface)
	local netmd = require "luci.model.network".init()
	local net = netmd:get_network(iface)
	local cmp=iface:match("openvpn")
	if net then
		if cmp == "openvpn" then
		luci.sys.exec("val=`uci show openvpn | grep %q | grep val | cut -d . -f2` ; ps | grep openvpn | grep $val | awk '{print$1}' | xargs kill -9" % iface)	
		end
		luci.sys.exec("uci set network.%q.stop=yes && uci commit network" % iface)
		luci.sys.call("env -i /sbin/ifdown %q >/dev/null 2>/dev/null" % iface)
		luci.http.status(200, "Shutdown")
		return
	end
	luci.http.status(404, "No such interface")
end

function iface_status(ifaces)
	local netm = require "luci.model.network".init()
	local rv   = { }
	
	local iface
	for iface in ifaces:gmatch("[%w%.%-_]+") do
		local cmp=iface:match("gre")
		local val=0
		if cmp=="gre" then
			state=luci.sys.exec("uci get network.%q.stop" % iface)
			if state and state ~= "yes" then 
				val=luci.sys.exec("uci get network.%q.rtunaddr |  awk -F'/' '{print$1}' | xargs ping -c 1 -t 5 | grep received | cut -d ',' -f2 | awk '{print$1}'" % iface )
				if val and tonumber(val) < 1  then
					luci.sys.exec("echo %q >> /tmp/gre1" % val)
					luci.sys.exec("ifup %q" % iface )  
				end	
			end
		end
		local net = netm:get_network(iface)
		local device = net and net:get_interface()
		if device then
			local data = {
			id         = iface,
			proto      = net:proto(),
			uptime     = net:uptime(),
			gwaddr     = net:gwaddr(),
			dnsaddrs   = net:dnsaddrs(),
			name       = device:shortname(),
			type       = device:type(),
			ifname     = device:name(),
			macaddr    = device:mac(),
			is_up      = device:is_up(),
			rx_bytes   = device:rx_bytes(),
			tx_bytes   = device:tx_bytes(),
			rx_packets = device:rx_packets(),
			tx_packets = device:tx_packets(),
			val        = val,
			
			ipaddrs    = { },
			ip6addrs   = { },
			subdevices = { }
		}

		local _, a
		for _, a in ipairs(device:ipaddrs()) do
			data.ipaddrs[#data.ipaddrs+1] = {
			addr      = a:host():string(),
			netmask   = a:mask():string(),
			prefix    = a:prefix()
		}
	end
	for _, a in ipairs(device:ip6addrs()) do
		if not a:is6linklocal() then
			data.ip6addrs[#data.ip6addrs+1] = {
			addr      = a:host():string(),
			netmask   = a:mask():string(),
			prefix    = a:prefix()
		}
	end
end

for _, device in ipairs(net:get_interfaces() or {}) do
	data.subdevices[#data.subdevices+1] = {
	name       = device:shortname(),
	type       = device:type(),
	ifname     = device:name(),
	macaddr    = device:mac(),
	macaddr    = device:mac(),
	is_up      = device:is_up(),
	rx_bytes   = device:rx_bytes(),
	tx_bytes   = device:tx_bytes(),
	rx_packets = device:rx_packets(),
	tx_packets = device:tx_packets(),
}
end

rv[#rv+1] = data
else
	rv[#rv+1] = {
	id   = iface,
	name = iface,
	type = "ethernet"
}
end
end

if #rv > 0 then
	luci.http.prepare_content("application/json")
	luci.http.write_json(rv)
	return
end

luci.http.status(404, "No such device")
end

function iface_delete(iface)
	local netmd = require "luci.model.network".init()
	local net = netmd:del_network(iface)
		local cmp=iface:match("openvpn")
	if net then
		if cmp == "openvpn" then
		luci.sys.exec("val=`uci show openvpn | grep %q | grep val | cut -d . -f2` ;  uci delete openvpn.$val;uci commit openvpn" % iface)	
		end
		luci.sys.call("env -i /sbin/ifdown %q >/dev/null 2>/dev/null" % iface)
		luci.http.redirect(luci.dispatcher.build_url("admin/services/VPN"))
		netmd:commit("network")
		netmd:commit("wireless")
		return
	end

	luci.http.status(404, "No such interface")
end
