local fs = require "nixio.fs"
local ut = require "luci.util"
local pt = require "luci.tools.proto"
local nw = require "luci.model.network"
local fw = require "luci.model.firewall"
local has_firewall = fs.access("/etc/config/firewall")

m = Map("network","")
if has_firewall then
        m:chain("firewall")
end
nw.init(m.uci)
fw.init(m.uci)

luci.sys.exec("mkdir -p /tmp/pptp")
local p = m:section( SimpleSection )
p.template = "vpn/pptp_pg"
a=luci.http.formvalue("val")
if a then
if (a == "0") then
local val = luci.sys.exec("cat /etc/config/network | grep config | grep -o pptp[0-9] | grep -o [0-9] | tail -1")
if val == "" then
val=1
a="pptp"..val
local d=io.open("/etc/config/network","a+")
d:write("\nconfig interface".." ".."'"..a.."'")
d:write("\n\toption proto 'pptp'")
d:write("\n\toption ifname".." ".."'"..a.."'")
d:write("\n\toption stop 'no'")
d:write("\n\toption metric".." ".."'"..val.."'")
d:close()
else
val=tonumber(val)+1
a="pptp"..val
local d=io.open("/etc/config/network","a+")
d:write("\nconfig interface".." ".."'"..a.."'")
d:write("\n\toption proto 'pptp'")
d:write("\n\toption ifname".." ".."'"..a.."'")
d:write("\n\toption stop 'no'")
d:write("\n\toption metric".." ".."'"..val.."'")
d:close()
end
end
section = m:section(NamedSection,a,a,nil)                                                                          
section.anonymous = true            
section.addremove = false
cmd="echo "..a.." > /tmp/pptp/pptp"
luci.sys.exec(cmd)
else
local d=io.open("/tmp/pptp/pptp","a+")
cd=d:read()
d:close()
section = m:section(NamedSection,cd,cd,nil)                                                                          
section.anonymous = true            
section.addremove = false
end  

section:tab("general",  translate("General Settings"))
section:tab("advanced",  translate("Advanced Settings"))
if has_firewall then
        section:tab("firewall", translate("Firewall Settings"))
end


local defaultroute, metric, peerdns, dns,
	keepalive_failure, keepalive_interval, demand, mtu

iface = section:taboption("general", ListValue, "interface", translate("Interface"))
ifc=luci.sys.exec("mwan3 interfaces | grep -v -e '^$' | grep -v pptp | grep -v status | grep -v openvpn | awk '{print$2}' && echo Auto")

i=1
for val in ifc:gmatch("(%S+)")   do
iface:value(val,val)      
if val ~= "Auto" then
e = section:taboption("general", DummyValue, "leftaddr"..i, "Address")
e.value=luci.sys.exec("ifstatus %q |  grep -A2 ipv4-address |grep -e \"\"address\"\" | cut -d: -f2 | cut -d '\"' -f2 | cut -d[ -f2  " % val)
e:depends({interface=val})
i=i+1
end
end


e=section:taboption("general", ListValue, "policy", "Policy")                                                                      
e:depends({interface="Auto"})
polcy=luci.sys.exec("mwan3 policies | grep Policy | awk '{print$2}' | cut -d ':' -f1")                                                  
for val1 in polcy:gmatch("(%S+)")   do
e:value(val1,val1)
end

server = section:taboption("general", Value, "server", translate("VPN Server"))
section.datatype = "host"

username = section:taboption("general", Value, "username", translate("PAP/CHAP username"))


password = section:taboption("general", Value, "password", translate("PAP/CHAP password"))
password.password = true

o= section:taboption("general",DummyValue,"","")
--o.template = "vpn/ipsec"

--boot=section:taboption("advanced", Flag, "auto", translate("Bring up on boot"))

defaultroute = section:taboption("advanced", Flag, "defaultroute",
	translate("Use default gateway"),
	translate("If unchecked, no default route is configured"))

defaultroute.default = defaultroute.enabled

mppe = section:taboption("advanced", Flag, "mppe",                                                                          
        translate("Use MPPE"))                                                                                                   

mppe.default = mppe.enabled

metric = section:taboption("advanced", Value, "metric",
	translate("Use gateway metric"))

metric.placeholder = "0"
metric.datatype    = "uinteger"
metric:depends("defaultroute", defaultroute.enabled)


peerdns = section:taboption("advanced", Flag, "peerdns",
	translate("Use DNS servers advertised by peer"),
	translate("If unchecked, the advertised DNS server addresses are ignored"))

peerdns.default = peerdns.enabled


dns = section:taboption("advanced", DynamicList, "dns",
	translate("Use custom DNS servers"))

dns:depends("peerdns", "")
dns.datatype = "ipaddr"
dns.cast     = "string"


keepalive_failure = section:taboption("advanced", Value, "_keepalive_failure",
	translate("LCP echo failure threshold"),
	translate("Presume peer to be dead after given amount of LCP echo failures, use 0 to ignore failures"))

function keepalive_failure.cfgvalue(self, section)
	local v = m:get(section, "keepalive")
	if v and #v > 0 then
		return tonumber(v:match("^(%d+)[ ,]+%d+") or v)
	end
end

function keepalive_failure.write() end
function keepalive_failure.remove() end

keepalive_failure.placeholder = "0"
keepalive_failure.datatype    = "uinteger"


keepalive_interval = section:taboption("advanced", Value, "_keepalive_interval",
	translate("LCP echo interval"),
	translate("Send LCP echo requests at the given interval in seconds, only effective in conjunction with failure threshold"))

function keepalive_interval.cfgvalue(self, section)
	local v = m:get(section, "keepalive")
	if v and #v > 0 then
		return tonumber(v:match("^%d+[ ,]+(%d+)"))
	end
end

function keepalive_interval.write(self, section, value)
	local f = tonumber(keepalive_failure:formvalue(section)) or 0
	local i = tonumber(value) or 5
	if i < 1 then i = 1 end
	if f > 0 then
		m:set(section, "keepalive", "%d %d" %{ f, i })
	else
		m:del(section, "keepalive")
	end
end

keepalive_interval.remove      = keepalive_interval.write
keepalive_interval.placeholder = "5"
keepalive_interval.datatype    = "min(1)"


demand = section:taboption("advanced", Value, "demand",
	translate("Inactivity timeout"),
	translate("Close inactive connection after the given amount of seconds, use 0 to persist connection"))

demand.placeholder = "0"
demand.datatype    = "uinteger"


mtu = section:taboption("advanced", Value, "mtu", translate("Override MTU"))
mtu.placeholder = "1500"
mtu.datatype    = "max(9200)"

o= section:taboption("advanced", DummyValue,"","")
--o.template = "vpn/ipsec"

if has_firewall then
        fwzone = section:taboption("firewall", Value, "_fwzone",
                translate("Create / Assign firewall-zone"),
                translate("Choose the firewall zone you want to assign to this interface. Select <em>unspecified</em> to remove the interface from the associated zone."))

        fwzone.template = "cbi/firewall_zonelist"
        fwzone.network = a
        fwzone.rmempty = false

               function fwzone.cfgvalue(self, section)
                self.iface = section
                luci.sys.exec("echo %q >> /tmp/ntfi" % section)
                local z = fw:get_zone_by_network(section)
                return z and z:name()
        end


              function fwzone.write(self, section, value)
                local zone = fw:get_zone(value)
                luci.sys.exec("echo %q >> /tmp/ntfi" % value)
                if not zone and value == '-' then
                        value = m:formvalue(self:cbid(section) .. ".newzone")
                        if value and #value > 0 then
                                zone = fw:add_zone(value)
                        else
                                fw:del_network(section)
                        end
                end

                if zone then
                        fw:del_network(section)
                        zone:add_network(section)
                end
        end


end



o= section:taboption("firewall", DummyValue,"","")
--o.template = "vpn/ipsec"


return m



