#!/bin/sh
[ $# -ne 3 ] && echo fail && exit 1

vpn=`echo "pptp openvpn l2tp ipsec gre" | grep -iow "$1"`
[ ! -n "${vpn}" ] && echo fail && exit 1 

startstop=`echo "start stop" | grep -iow "$3"`
[ ! -n "${startstop}" ] && echo fail && exit 1

case ${vpn} in
	l2tp)
	{
		name=`echo $2 | tr 'A-Z' 'a-z'`
		interface=`uci get network.$name 2>/dev/null`
		[ -z $interface ] && echo fail && exit 1
		if [ "$startstop" == "start" ]; then 
			uci set network.$name.stop=no
			uci commit
			ifup $name
		elif [ "$startstop" == "stop" ]; then 
			uci set network.$name.stop=yes
			uci commit    
			ifdown $name
		fi
	}
	;;
	pptp)
	{
		name=`echo $2 | tr 'A-Z' 'a-z'`
		interface=`uci get network.$name 2>/dev/null`
		[ -z $interface ] && echo fail && exit 1
		if [ "$startstop" == "start" ]; then 
			uci set network.$name.stop=no
			uci commit
			ifup $name
		elif [ "$startstop" == "stop" ]; then 
			uci set network.$name.stop=yes
			uci commit
			ifdown $name
		fi
	}
	;;
	ipsec)
	{
		name=`echo $2 | tr 'a-z' 'A-Z'`
		interface=`uci get ipsec.$name 2>/dev/null`
		[ -z $interface ] && echo fail && exit 1
		if [ "$startstop" == "start" ]; then 
			uci set ipsec.$name.enable=1
			uci commit
			/sbin/ipsec1 update &>/dev/null
			/sbin/ipsec1 Reconnect "`uci get ipsec.$name.conn`" "$name" &>/dev/null
		elif [ "$startstop" == "stop" ];then 
			uci set ipsec.$name.enable=0
			uci commit
			/sbin/ipsec1 Disconnect "`uci get ipsec.$name.conn`" "$name" &>/dev/null
		elif [ "$startstop" == "startserver" ];then 
			uci set ipsec.$name.enable=1
			uci commit
			/sbin/ipsec1 update &>/dev/null
			/sbin/ipsec1 StartSrv "`uci get ipsec.$name.conn`" "$name" &>/dev/null
		elif [ "$startstop" == "stopserver" ];then 
			uci set ipsec.$name.enable=0
			uci commit
			/sbin/ipsec1 StopSrv "`uci get ipsec.$name.conn`" "$name" &>/dev/null
		fi
	}
	;;
	openvpn)
	{
		name=$2
		[ -z $name ] && echo fail && exit 1 
		interface=`uci get openvpn.$name 2>/dev/null`
		[ -z $interface ] && echo fail && exit 1 
		if [ "$startstop" == "start" ]; then 
			uci set openvpn.$name.enable=1
			uci commit
			/etc/init.d/openvpn start $name &>/dev/null
		elif [ "$startstop" == "stop" ]; then 
			uci set openvpn.$name.enable=0
			uci commit
			rm -f /tmp/etc/openvpn-$name.conf
			local pid=`ps |grep "syslog openvpn($name)" | grep -v grep | awk '{print $1}'`
			if [ "$pid" != "" ];then
				kill -15 $pid
			fi
			/etc/init.d/openvpn stop openvpn_$name &>/dev/null
		fi
	}
	;;
	gre)
	{
		name=`echo $2 | tr 'A-Z' 'a-z'`
		interface=`uci get network.$name 2>/dev/null`
		[ -z $interface ] && echo fail && exit 1
		if [ "$startstop" == "start" ]; then 
			uci set network.$name.stop=no
			uci commit
			ifup $name
		elif [ "$startstop" == "stop" ]; then 
			uci set network.$name.stop=yes
			uci commit
			ifdown $name
		fi
	}
	;;
esac


echo pass

exit 0
